'use strict';
const log = require('@foundation-cli/log');
const semver = require('semver'); // 比对版本的模块
const colors = require('colors/safe');
const LOWEST_NODE_VERSION = '12.0.0'; // node的最低版本

class Command {
  constructor(argv) {
    if (!argv) {
      throw new Error('参数不能为空');
    }
    if (!Array.isArray(argv)) {
      throw new Error('参数必须为数组');
    }
    if (!argv.length) {
      throw new Error('参数列表不能为空');
    }

    this._argv = argv;

    // 构建微任务队列
    let runner = new Promise((resolve, reject) => {
      let chain = Promise.resolve();

      chain = chain.then(() => {
        this.checkNodeVersion();
      });
      chain = chain.then(() => {
        this.initArgs();
      });
      chain = chain.then(async () => {
        await this.init();
      });
      chain = chain.then(async () => {
        await this.exec();
      });

      chain.catch((e) => { // 处理错误
        log.error(e.message);
      })
    });
  }
  // 检查当前node的版本号
  checkNodeVersion() {
    // 第一步：使用process.version获取当前node版本号
    const currentNodeVersion = process.version;

    // 第二步：比对最低版本号
    const lowestNodeVersion = LOWEST_NODE_VERSION;

    if (!semver.gte(currentNodeVersion, lowestNodeVersion)) {
      // colors.red可以将定义的错误信息显示为红色
      throw new Error(colors.red(`foundation-cli 需要安装v${lowestNodeVersion} 以上版本的node.js`));
    }

    log.info('current_node_version', currentNodeVersion);
  }
  initArgs() {
    this.cmd = this._argv[this._argv.length - 1]; // 获取commander库的command实例
    this._argv = this._argv.slice(0, this._argv.length - 1);
  }
  init() {
    throw new Error('子类必须自己实现init方法');
  }
  exec() {
    throw new Error('子类必须自己实现exec方法');
  }
}

module.exports = Command;
